@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2019-2020 Andreas Weber <octave@josoansi.de>
+  Copyright (C) 2019-2023 Andreas Weber <octave@josoansi.de>
 
   This file is part of octave-video; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -17,6 +17,21 @@
 
 #include "cap_ffmpeg_impl_ov.hpp"
 
+// PKG_ADD: autoload ("__octave_video_set_verbosity_level__", "cap_ffmpeg_wrapper.oct");
+// PKG_DEL: autoload ("__octave_video_set_verbosity_level__", "cap_ffmpeg_wrapper.oct", "remove");
+DEFUN_DLD(__octave_video_set_verbosity_level__, args, nargout,
+          "-*- texinfo -*-\n\
+@deftypefn {Loadable Function} {@var{def} =} __octave_video_set_verbosity_level__ ()\n\
+undocumented internal function\n\
+@end deftypefn")
+{
+  // default = 1 => show errors and warnings
+  octave_value_list retval;
+  int l = args(0).int_value ();
+  set_verbosity_level (l);
+  return retval;
+}
+
 // PKG_ADD: autoload ("__ffmpeg_defines__", "cap_ffmpeg_wrapper.oct");
 // PKG_DEL: autoload ("__ffmpeg_defines__", "cap_ffmpeg_wrapper.oct", "remove");
 DEFUN_DLD(__ffmpeg_defines__, args, nargout,
@@ -40,11 +55,9 @@ undocumented internal function\n\
   opt.contents ("LIBAVFORMAT_BUILD") = LIBAVFORMAT_BUILD;
   opt.contents ("LIBAVFORMAT_IDENT") = LIBAVFORMAT_IDENT;
 
-
   //join ident
   opt.contents ("LIBAV_IDENT") = LIBAVUTIL_IDENT ", " LIBSWSCALE_IDENT ", " LIBAVCODEC_IDENT ", " LIBAVFORMAT_IDENT;
 
-
   retval.append (opt);
 
   return retval;
@@ -58,17 +71,12 @@ DEFUN_DLD(__ffmpeg_output_formats__, args, nargout,
 undocumented internal function\n\
 @end deftypefn")
 {
-  av_register_all();
-
   octave_idx_type n = 0;
 
   // first loop to get numer of output formats
-  AVOutputFormat * oformat = av_oformat_next(NULL);
-  while (oformat != NULL)
-    {
-      n++;
-      oformat = av_oformat_next (oformat);
-    }
+  const AVOutputFormat * oformat;
+  void *opaque = NULL;
+  while ((oformat = av_muxer_iterate(&opaque))) {n++;}
 
   Cell names (n, 1);
   Cell long_names (n, 1);
@@ -77,9 +85,9 @@ undocumented internal function\n\
   Cell codecs (n, 1);
 
   // second loop, now fill the cells
-  oformat = av_oformat_next(NULL);
   int i = 0;
-  while(oformat != NULL)
+  opaque = NULL;
+  while ((oformat = av_muxer_iterate(&opaque)))
     {
       names (i) = oformat->name;
       long_names (i) = oformat->long_name;
@@ -140,8 +148,6 @@ undocumented internal function\n\
             codecs (i) = codec_fourcc;
           }
         }
-
-      oformat = av_oformat_next(oformat);
       i++;
     }
 
@@ -224,6 +230,7 @@ Gets CvCapture_FFMPEG properties like bitrate, fps, total_frames, duration_sec..
   CvCapture_FFMPEG* h = get_cap_from_ov (args(0));
   if (h)
     {
+
       octave_scalar_map opt;
       opt.contents ("total_frames") = h->get_total_frames ();
       opt.contents ("duration_sec") = h->get_duration_sec ();
@@ -251,6 +258,8 @@ Gets CvCapture_FFMPEG properties like bitrate, fps, total_frames, duration_sec..
         opt.contents ("aspect_ration_den") = s.den;
       }
 
+      opt.contents ("is_okay") = h->is_ok();
+
       retval.append (opt);
     }
   return retval;
@@ -299,10 +308,11 @@ DEFUN_DLD(__cap_retrieve_frame__, args, nargout,
       int height = 0;
       int step;         // AVFrame::linesize, size in bytes of each picture line
       int cn;           // number of colors and should always be 3 here
+      int depth;        // 0 = CV_8U, 1 glaube ich signed, 2 = CV_16U
 
-      bool ret = p->retrieveFrame (0, &data, &step, &width, &height, &cn);
+      bool ret = p->retrieveFrame (0, &data, &step, &width, &height, &cn, &depth);
 
-      //printf ("ret = %i, width = %i, height = %i, step = %i, cn = %i\n", ret, width, height, step, cn);
+      //printf ("ret = %i, width = %i, height = %i, step = %i, cn = %i, depth = %i\n", ret, width, height, step, cn, depth);
 
       assert (cn == 3);
 
@@ -413,7 +423,6 @@ undocumented internal function\n\
     {
       CvVideoWriter_FFMPEG::register_type();
       writer_type_loaded = true;
-      av_register_all();
     }
 
   std::string filename = args(0).string_value ();
@@ -432,23 +441,30 @@ undocumented internal function\n\
   if (fourcc.size () == 0)
     {
       // get tag for default codec for guessed container from filename
-      AVOutputFormat* foo = av_guess_format	(NULL, filename.c_str (), NULL);
+      const AVOutputFormat* fmt = av_guess_format	(NULL, filename.c_str (), NULL);
+
+      if (! fmt)
+      {
+        error ("Can't guess container format from filename '%s'", filename.c_str ());
+        return retval;
+      }
+
+      MSG_INFO ("Guessed format '%s' from filename '%s'", fmt->long_name, filename.c_str ());
 
       // list supported codecs for guessed format
-#if 0
-      if (foo->codec_tag)
+      if (fmt && fmt->codec_tag)
         {
-          const AVCodecTag * ptags = foo->codec_tag[0];
+          MSG_VERBOSE ("supported codecs for format '%s':", fmt->long_name);
+          const AVCodecTag * ptags = fmt->codec_tag[0];
           while (ptags->id != AV_CODEC_ID_NONE)
             {
               unsigned int tag = ptags->tag;
-              printf("fourcc tag 0x%08x/'%c%c%c%c' codec_id %04X\n", tag, CV_TAG_TO_PRINTABLE_CHAR4(tag), ptags->id);
+              MSG_VERBOSE("  fourcc tag 0x%08x/'%c%c%c%c' codec_id %04X", tag, CV_TAG_TO_PRINTABLE_CHAR4(tag), ptags->id);
               ptags++;
             }
         }
-#endif
 
-      tag = av_codec_get_tag (foo->codec_tag, foo->video_codec);
+      tag = av_codec_get_tag (fmt->codec_tag, fmt->video_codec);
     }
   else if (fourcc.size () == 4)
     {
@@ -457,6 +473,8 @@ undocumented internal function\n\
   else
     error ("fourcc has to be empty or 4 chars long");
 
+  MSG_INFO ("selected tag = %i = %#x = '%c%c%c%c'", tag, tag, CV_TAG_TO_PRINTABLE_CHAR4(tag));
+
   // list codecs
   //~ AVCodec * codec = av_codec_next(NULL);
   //~ while(codec != NULL)
@@ -494,8 +512,6 @@ undocumented internal function\n\
   //~ oformat = av_oformat_next(oformat);
   //~ }
 
-  //printf ("tag = %i = %#x = %c%c%c%c\n", tag, tag, CV_TAG_TO_PRINTABLE_CHAR4(tag));
-
 #if 0
   // that would be a workaround:
   AVOutputFormat* foo = av_guess_format	(NULL, "foo.mp4", NULL);
@@ -537,9 +553,9 @@ undocumented internal function\n\
   // FFMPEG backend with MP4 container natively uses other values as fourcc code: see http://mp4ra.org/#/codecs,
   // so you may receive a warning message from OpenCV about fourcc code conversion.
 
-  // fps	Framerate of the created video stream.
-  // isColor	If it is not zero, the encoder will expect and encode color frames,
-  // otherwise it will work with grayscale frames (the flag is currently supported on Windows only).
+  // fps	    Framerate of the created video stream.
+  // isColor    If it is not zero, the encoder will expect and encode color frames,
+  // otherwise  it will work with grayscale frames (the flag is currently supported on Windows only).
 
   //printf ("h->open (%s, %i, %f, %u, %u, %u);\n", filename.c_str (), tag, fps, width, height, isColor);
 
